/********************************************************************** 
 Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <TextEdit.h>
#include <Fonts.h>
#include <Types.h>

#include "mem.h"
#include "packets.h"

#include "log.h"
#include "clinet.h"

#include "chatline.h"

#include "shared.h"

DialogPtr chat_dialog;
TEHandle chat_text;
Ptr chat_storage;
pascal void update_output_window(DialogPtr dlog, short item);

/**************************************************************************
...
**************************************************************************/
void inputline_return(void)
{
  struct packet_generic_message apacket;
  short the_type;
  Handle the_handle;
  Rect the_rect;
  Str255 the_input;
  
  GetDItem( chat_dialog, 1, &the_type, & the_handle, &the_rect );
  GetIText( the_handle, the_input);
    
  if(the_input) {
    strncpy(apacket.message, (const char *)the_input, MSG_SIZE-NAME_SIZE);
    apacket.message[MSG_SIZE-NAME_SIZE]='\0';
    send_packet_generic_message(&aconnection, PACKET_CHAT_MSG, &apacket);
  }
  SetIText( the_handle, "\p\0");
}

void append_output_window(char *astring)
{
  long loc=strlen(*TEGetText(chat_text));
  strcat(astring, (const char *)'\n');
  TESetSelect(loc,loc, chat_text);
  TEInsert(astring, strlen(astring), chat_text);
}

/**************************************************************************
 I have no idea what module this belongs in -- Syela
 I've decided to put output_window routines in chatline.c, because
 the are somewhat related and append_output_window is already here.  --dwp
**************************************************************************/
void log_output_window(void)
{ 
  StandardFileReply reply;
  OSErr err;
  StandardPutFile((const unsigned char *)"Save chatline as...",
                  (const unsigned char *)"civgame.log",&reply);
  if(reply.sfGood)
  {
    const FSSpec file=reply.sfFile;
    err = FSpCreate(&file, 'ttxt', 'TEXT', reply.sfScript);
    if(err == 0){
      short ref_num;
      err = FSpOpenDF(&file, fsWrPerm, &ref_num);
      if(err == 0){
        char *the_output;
        long len;
        append_output_window("Exporting output window to civgame.log ...");
        the_output = *TEGetText(chat_text);
        len=strlen(the_output);
        err=FSWrite(ref_num, &len, the_output);
        if (err != 0){
          append_output_window("Export failed.");
          freelog(LOG_NORMAL, "Error in log_output_window (FSWrite failed)");
        }else
          append_output_window("Export complete.");
        err=FSClose(ref_num);
        if (err != 0){
          append_output_window("Error Closing File");
          freelog(LOG_NORMAL, "Error in log_output_window (FSClose failed)");
          /*exit(1)*/ /*should the program bomb on this error?*/
        }else{
          err=FlushVol(nil, file.vRefNum);
          if(err != 0)
            freelog(LOG_NORMAL, "Error in log_output_window (FlushVol failed)");
            /*should this bomb or tell the user an error happened?*/
        }
      }else{
        append_output_window("Error opening output file");
        freelog(LOG_NORMAL, "Error in log_output_window (FSpOpenDF failed)");
      }
    }else{
      append_output_window("Error creating output file");
      freelog(LOG_NORMAL, "Error in log_output_window (FSpCreate failed)");
    }
  }
}

/**************************************************************************
...
**************************************************************************/
void clear_output_window(void)
{
  char** handle=TEGetText(chat_text);
  char* new_str="Cleared output window.\n";
  TESetText(&new_str, strlen(new_str), chat_text);
  DisposeHandle(handle);
}

/**************************************************************************
... Begin mac sections
**************************************************************************/

void init_output_window(void)
{
  short the_type;
  Handle the_handle;
  Rect the_rect;
  GrafPtr port;
  short font;
  short size;
  Style style;  
  FontInfo finfo;
  
  chat_storage = NewPtr(sizeof(DialogRecord));
  if (!chat_storage)
  {
    freelog(LOG_FATAL, "Error initizing chat window (NewPtr failed)");
    exit(1);
  }
  chat_dialog = GetNewDialog(kChatDITL, chat_storage, (WindowPtr)-1L);
  if (!chat_storage)
  {
    freelog(LOG_FATAL, "Error initizing chat window (GetNewDialog failed)");
    exit(1);
  }
  
  GetDItem( chat_dialog, 2, &the_type, &the_handle, &the_rect );
  
  chat_text=TENew(&the_rect, &the_rect);
  if (!chat_text)
  {
    freelog(LOG_FATAL, "Error initizing chat window (TENew failed)");
    exit(1);
  }
  GetPort(&port);
  SetPort((**chat_text).inPort);

  font = (**chat_text).inPort->txFont;
  size = (**chat_text).inPort->txSize;
  style = (**chat_text).inPort->txFace;

  TextFont((**chat_text).txFont=geneva);
  TextSize((**chat_text).txSize=12);
  TextFace((**chat_text).txFace=normal);

  GetFontInfo(&finfo);

  TextFont(font);
  TextSize(size);
  TextFace(style);
  SetPort(port);
  (**chat_text).lineHeight=finfo.ascent+finfo.descent+finfo.leading;
  (**chat_text).fontAscent=finfo.ascent;
  
  SetDItem( chat_dialog, 2, the_type, (Handle)update_output_window, &the_rect );
}

pascal void update_output_window(DialogPtr dlog, short item)
{
  if ((dlog !=chat_dialog) || (item != 2)) /*verify this is the right update*/
  {
    short the_type;
    Handle the_handle;
    Rect the_rect;
    GetDItem( chat_dialog, 2, &the_type, &the_handle, &the_rect );
    TEUpdate(&the_rect, chat_text); /*this is suposed to redraw the whole TE window*/
  }
}

void close_output_window(void)
{
  TEDispose(chat_text);
  CloseDialog(chat_dialog);
  DisposePtr(chat_storage);
}