/********************************************************************** 
 Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/
#include <string.h>
#include <bool.h>

#include "capability.h"
#include "log.h"
#include "map.h"
#include "unit.h"

#include "chatline.h"
#include "cityrep.h"
#include "civclient.h"
#include "clinet.h"
#include "dialogs.h"
#include "finddlg.h"
#include "gotodlg.h"
#include "gui_stuff.h"
#include "helpdlg.h"
#include "mapctrl.h"
#include "messagedlg.h"
#include "meswindlg.h"
#include "optiondlg.h"
#include "options.h"
#include "plrdlg.h"
#include "ratesdlg.h"
#include "repodlgs.h"
#include "spaceshipdlg.h"

#include "menu.h"

/* I moved one menu item to accomidate apple's user interface guidelines.
// That menu item was the quit command in the game menu.
// It was moved to the File Menu.  -AWB*/

#include <Traps.h>	/* I think these are the needed mac headers */
#include <GestaltEqu.h>

extern bool done;

/****************************************************************
...
*****************************************************************/
enum unit_activity road_activity;

enum MenuID {
  MENU_APPLE=0,
  MENU_APPLE_ABOUT,
  MENU_APPLE_SEP1,
  MENU_FILE=0,
  MENU_FILE_QUIT,
  MENU_EDIT=0,
  MENU_EDIT_CUT,
  MENU_EDIT_COPY,
  MENU_EDIT_PASTE,
  MENU_EDIT_SEP1,
  MENU_EDIT_UNDO,
  MENU_GAME=0,
  MENU_GAME_OPTIONS,
  MENU_GAME_MSG_OPTIONS,
  MENU_GAME_SAVE_SETTINGS,
  MENU_GAME_SEP1,
  MENU_GAME_PLAYERS,
  MENU_GAME_MESSAGES,
  MENU_GAME_SERVER_OPTIONS1,
  MENU_GAME_SERVER_OPTIONS2,
  MENU_GAME_OUTPUT_LOG,
  MENU_GAME_CLEAR_OUTPUT,
  MENU_GAME_DISCONNECT,
  MENU_KINGDOM=0,
  MENU_KINGDOM_TAX_RATE,
  MENU_KINGDOM_SEP1,
  MENU_KINGDOM_FIND_CITY,
  MENU_KINGDOM_SEP2,
  MENU_KINGDOM_REVOLUTION,
  MENU_VIEW=0,
  MENU_VIEW_SHOW_MAP_GRID,
  MENU_VIEW_CENTER_VIEW,
  MENU_ORDER=0,
  MENU_ORDER_AUTO_SETTLER,
  MENU_ORDER_AUTO_ATTACK,
  MENU_ORDER_CITY,
  MENU_ORDER_ROAD,
  MENU_ORDER_IRRIGATE,
  MENU_ORDER_TRANSFORM,
  MENU_ORDER_MINE,
  MENU_ORDER_SEP1,
  MENU_ORDER_FORTRESS,
  MENU_ORDER_POLLUTION,
  MENU_ORDER_SEP2,
  MENU_ORDER_EXPLORE,
  MENU_ORDER_NUKE,
  MENU_ORDER_UNLOAD,
  MENU_ORDER_GOTO,
  MENU_ORDER_GOTO_CITY,
  MENU_ORDER_HOMECITY,
  MENU_ORDER_SEP3,
  MENU_ORDER_FORTIFY,
  MENU_ORDER_SENTRY,
  MENU_ORDER_SEP4,
  MENU_ORDER_DISBAND,
  MENU_ORDER_WAKEUP,
  MENU_ORDER_WAIT,
  MENU_ORDER_PILLAGE,
  MENU_ORDER_BUILD_WONDER,
  MENU_ORDER_TRADE_ROUTE,
  MENU_ORDER_DONE,
  MENU_REPORT=0,
  MENU_REPORT_CITY,
  MENU_REPORT_SCIENCE,
  MENU_REPORT_TRADE,
  MENU_REPORT_ACTIVE_UNITS,
  MENU_REPORT_SEP1,
  MENU_REPORT_WOW,
  MENU_REPORT_TOP_CITIES,
  MENU_REPORT_DEMOGRAPHIC,
  MENU_REPORT_SPACESHIP,
  MENU_HELP=0,
  MENU_HELP_CONNECTING,
  MENU_HELP_CONTROLS,
  MENU_HELP_CHATLINE,
  MENU_HELP_PLAYING,
  MENU_HELP_IMPROVEMENTS,
  MENU_HELP_UNITS,
  MENU_HELP_COMBAT,
  MENU_HELP_ZOC,
  MENU_HELP_TECH,
  MENU_HELP_TERRAIN,
  MENU_HELP_WONDERS,
  MENU_HELP_GOVERNMENT,
  MENU_HELP_HAPPINESS,
  MENU_HELP_SPACE_RACE,
  MENU_HELP_COPYING,
  MENU_HELP_ABOUT
};

MenuHandle apple_menu, file_menu, edit_menu, game_menu, kingdom_menu,
           view_menu, order_menu, report_menu, help_menu;
    /*handles for the mac menus*/

void apple_menu_callback(short menu_item);
void file_menu_callback(short menu_item);
void edit_menu_callback(short menu_item);
void game_menu_callback(short menu_item);
void kingdom_menu_callback(short menu_item);
void view_menu_callback(short menu_item);
void orders_menu_callback(short menu_item);
void reports_menu_callback(short menu_item);
void help_menu_callback(short menu_item);

void menu_callback(long hit)
{
  short menu = hit >> 16;
  short item = hit & 0xFFFF;
  switch(menu)
  {
    case 128:
      apple_menu_callback(item);
      break;
    case 129:
      file_menu_callback(item);
      break;
    case 130:
      edit_menu_callback(item);
      break;
    case 131:
      game_menu_callback(item);
      break;
    case 132:
      kingdom_menu_callback(item);
      break;
    case 133:
      view_menu_callback(item);
      break;
    case 134:
      order_menu_callback(item);
      break;
    case 135:
      report_menu_callback(item);
      break;
    case 136:
      help_menu_callback(item);
      break;
  }
  HiliteMenu(0);
}

void apple_menu_callback(short menu_item)
{
  Str255 desk_acc_name;
  int desk_acc_number;
  switch(menu_item){
  case MENU_APPLE_ABOUT:
    popup_help_dialog_string(HELP_ABOUT_ITEM);
    break;
  default:
    GetItem(apple_menu, menu_item, desk_acc_name);
    desk_acc_number=OpenDeskAcc(desk_acc_name);
    break;
  }
}   

/****************************************************************
...
*****************************************************************/
void game_menu_callback(short menu_item)
{
  switch(menu_item) {
  case MENU_GAME_OPTIONS:
    popup_option_dialog();
    break;
  case MENU_GAME_MSG_OPTIONS:
    popup_messageopt_dialog();
    break;
  case MENU_GAME_SAVE_SETTINGS:
    save_options();
    break;
  case MENU_GAME_PLAYERS:
    popup_players_dialog();
    break;
  case MENU_GAME_MESSAGES:
    popup_meswin_dialog();
    break;
  case MENU_GAME_SERVER_OPTIONS1:
    send_report_request(REPORT_SERVER_OPTIONS1);
    break;
  case MENU_GAME_SERVER_OPTIONS2:
    send_report_request(REPORT_SERVER_OPTIONS2);
    break;
  case MENU_GAME_OUTPUT_LOG:
    log_output_window();
    break;
  case MENU_GAME_CLEAR_OUTPUT:
    clear_output_window();
    break;
  case MENU_GAME_DISCONNECT:
    disconnect_from_server();
    break;
  }
}


/****************************************************************
...
*****************************************************************/
void kingdom_menu_callback(short menu_item)
{
  switch(menu_item) {
  case MENU_KINGDOM_TAX_RATE:
    popup_rates_dialog();
    break;
  case MENU_KINGDOM_FIND_CITY:
    popup_find_dialog();
    break;
  case MENU_KINGDOM_REVOLUTION:
    popup_revolution_dialog();
    break;
  }
}


/****************************************************************
...
*****************************************************************/
void view_menu_callback(short menu_item)
{
  switch(menu_item) {
  case MENU_VIEW_SHOW_MAP_GRID:
    key_map_grid ();
    break;
  case MENU_VIEW_CENTER_VIEW:
    center_on_unit();
    break;
  }
}


/****************************************************************
...
*****************************************************************/
void orders_menu_callback(short menu_item)
{
  switch(menu_item) {
   case MENU_ORDER_AUTO_SETTLER:
   case MENU_ORDER_AUTO_ATTACK:
    if(get_unit_in_focus())
      request_unit_auto(get_unit_in_focus());
    break;
   case MENU_ORDER_CITY:
    if(get_unit_in_focus())
      request_unit_build_city(get_unit_in_focus());
    break;
   case MENU_ORDER_IRRIGATE:
    if(get_unit_in_focus())
      request_new_unit_activity(get_unit_in_focus(), ACTIVITY_IRRIGATE);
    break;
   case MENU_ORDER_FORTRESS:
    if(get_unit_in_focus())
      request_new_unit_activity(get_unit_in_focus(), ACTIVITY_FORTRESS);
    break;
   case MENU_ORDER_MINE:
    if(get_unit_in_focus())
      request_new_unit_activity(get_unit_in_focus(), ACTIVITY_MINE);
    break;
   case MENU_ORDER_TRANSFORM:
    if(get_unit_in_focus())
      request_new_unit_activity(get_unit_in_focus(), ACTIVITY_TRANSFORM);
    break;
   case MENU_ORDER_ROAD:
    if(get_unit_in_focus())
      request_new_unit_activity(get_unit_in_focus(), road_activity);
    break;
   case MENU_ORDER_POLLUTION:
    if(get_unit_in_focus())
      request_new_unit_activity(get_unit_in_focus(), ACTIVITY_POLLUTION);
    break;
   case MENU_ORDER_HOMECITY:
    if(get_unit_in_focus())
      request_unit_change_homecity(get_unit_in_focus());
    break;
   case MENU_ORDER_FORTIFY:
    if(get_unit_in_focus())
      request_new_unit_activity(get_unit_in_focus(), ACTIVITY_FORTIFY);
    break;
   case MENU_ORDER_SENTRY:
    if(get_unit_in_focus())
      request_new_unit_activity(get_unit_in_focus(), ACTIVITY_SENTRY);
    break;
   case MENU_ORDER_WAIT:
    if(get_unit_in_focus())
      request_unit_wait(get_unit_in_focus());
    break;
   case MENU_ORDER_UNLOAD:
    if(get_unit_in_focus())
      request_unit_unload(get_unit_in_focus());
    break;
   case MENU_ORDER_WAKEUP:
    if(get_unit_in_focus())
      request_unit_wakeup(get_unit_in_focus());
    break;
   case MENU_ORDER_GOTO:
    if(get_unit_in_focus())
      request_unit_goto();
    break;
   case MENU_ORDER_GOTO_CITY:
    if(get_unit_in_focus())
      popup_goto_dialog();
    break;
   case MENU_ORDER_DISBAND:
    if(get_unit_in_focus())
      request_unit_disband(get_unit_in_focus());
    break;
   case MENU_ORDER_PILLAGE:
    if(get_unit_in_focus())
      request_new_unit_activity(get_unit_in_focus(), ACTIVITY_PILLAGE);
     break;
   case MENU_ORDER_EXPLORE:
    if(get_unit_in_focus())
      request_new_unit_activity(get_unit_in_focus(), ACTIVITY_EXPLORE);
     break;
   case MENU_ORDER_BUILD_WONDER:
    if(get_unit_in_focus())
      request_unit_caravan_action(get_unit_in_focus(),
				 PACKET_UNIT_HELP_BUILD_WONDER);
     break;
   case MENU_ORDER_TRADE_ROUTE:
    if(get_unit_in_focus())
      request_unit_caravan_action(get_unit_in_focus(),
				 PACKET_UNIT_ESTABLISH_TRADE);
     break;
   case MENU_ORDER_DONE:
    if(get_unit_in_focus())
      request_unit_move_done();
    break;
   case MENU_ORDER_NUKE:
    if(get_unit_in_focus())
      request_unit_nuke(get_unit_in_focus());
    break;
  }

}


/****************************************************************
...
*****************************************************************/
void reports_menu_callback(short menu_item)
{
  switch(menu_item) {
   case MENU_REPORT_CITY:
    popup_city_report_dialog(0);
    break;
   case MENU_REPORT_SCIENCE:
    popup_science_dialog(0);
    break;
   case MENU_REPORT_TRADE:
    popup_trade_report_dialog(0);
    break;
   case MENU_REPORT_ACTIVE_UNITS:
    popup_activeunits_report_dialog(0);
    break;
  case MENU_REPORT_DEMOGRAPHIC:
    send_report_request(REPORT_DEMOGRAPHIC);
    break;
  case MENU_REPORT_TOP_CITIES:
    send_report_request(REPORT_TOP_5_CITIES);
    break;
   case MENU_REPORT_WOW:
    send_report_request(REPORT_WONDERS_OF_THE_WORLD);
    break;
   case MENU_REPORT_SPACESHIP:
    popup_spaceship_dialog(game.player_ptr);
    break;
  }
}


/****************************************************************
...
*****************************************************************/
void help_menu_callback(short menu_item)
{
  switch(menu_item) {
  case MENU_HELP_CONNECTING:
    popup_help_dialog_string(HELP_CONNECTING_ITEM);
    break;
  case MENU_HELP_CHATLINE:
    popup_help_dialog_string(HELP_CHATLINE_ITEM);
    break;
  case MENU_HELP_CONTROLS:
    popup_help_dialog_string(HELP_CONTROLS_ITEM);
    break;
  case MENU_HELP_PLAYING:
    popup_help_dialog_string(HELP_PLAYING_ITEM);
    break;
  case MENU_HELP_IMPROVEMENTS:
    popup_help_dialog_string(HELP_IMPROVEMENTS_ITEM);
    break;
  case MENU_HELP_UNITS:
    popup_help_dialog_string(HELP_UNITS_ITEM);
    break;
  case MENU_HELP_COMBAT:
    popup_help_dialog_string(HELP_COMBAT_ITEM);
    break;
  case MENU_HELP_ZOC:
    popup_help_dialog_string(HELP_ZOC_ITEM);
    break;
  case MENU_HELP_TECH:
    popup_help_dialog_string(HELP_TECHS_ITEM);
    break;
  case MENU_HELP_TERRAIN:
    popup_help_dialog_string(HELP_TERRAIN_ITEM);
    break;
  case MENU_HELP_WONDERS:
    popup_help_dialog_string(HELP_WONDERS_ITEM);
    break;
  case MENU_HELP_GOVERNMENT:
    popup_help_dialog_string(HELP_GOVERNMENT_ITEM);
    break;
  case MENU_HELP_HAPPINESS:
    popup_help_dialog_string(HELP_HAPPINESS_ITEM);
    break;
  case MENU_HELP_SPACE_RACE:
    popup_help_dialog_string(HELP_SPACE_RACE_ITEM);
    break;
  case MENU_HELP_COPYING:
    popup_help_dialog_string(HELP_COPYING_ITEM);
    break;
  case MENU_HELP_ABOUT:
    popup_help_dialog_string(HELP_ABOUT_ITEM);
    break;
  }
}

void setup_menus(void)
{
  Handle mbar;
  mbar=GetNewMBar(128);
  if(mbar=0) {
    freelog(LOG_FATAL,
            "Couldn't load menubar (MBAR) resource!");
	exit(1);
  }
  SetMenuBar(mbar);
  DisposHandle(mbar);
  
  apple_menu=GetMHandle(128);
  file_menu=GetMHandle(129);
  edit_menu=GetMHandle(130);
  game_menu=GetMHandle(131);
  kingdom_menu=GetMHandle(132);
  view_menu=GetMHandle(133);
  order_menu=GetMHandle(134);
  report_menu=GetMHandle(135);
  help_menu=GetMHandle(136);
  AddResMenu(apple_menu, 'DRVR');
  DrawMenuBar();
}

void menus_set_sensitive(MenuHandle menu, int item, bool active)
{
  if (active)
    EnableItem(menu, item);
  else
    DisableItem(menu, item);
}

/****************************************************************
...
*****************************************************************/
void menus_rename(MenuHandle menu, int item, Str255 string)
{
  SetItem(menu, item, string);
}


/****************************************************************
...
*****************************************************************/
void update_menus(void)
{
  if(get_client_state()!=CLIENT_GAME_RUNNING_STATE) {
    menus_set_sensitive(report_menu,MENU_REPORT, false); /* <main>/Reports */
    menus_set_sensitive(kingdom_menu,MENU_KINGDOM, false); /* <main>/Kingdom */
    menus_set_sensitive(view_menu,MENU_VIEW, false); /* <main>/View */
    menus_set_sensitive(order_menu,MENU_ORDER, false); /* <main>/Orders */

    menus_set_sensitive(game_menu,MENU_GAME_OPTIONS, false); /* <main>/Game/Options */
    menus_set_sensitive(game_menu,MENU_GAME_MSG_OPTIONS, false); /* <main>/Game/Message Options */
    menus_set_sensitive(game_menu,MENU_GAME_SAVE_SETTINGS, false); /* <main>/Game/Save Settings */
    menus_set_sensitive(game_menu,MENU_GAME_PLAYERS, false); /* <main>/Game/Players */
    menus_set_sensitive(game_menu,MENU_GAME_MESSAGES, false); /* <main>/Game/Messages */
    menus_set_sensitive(game_menu,MENU_GAME_SERVER_OPTIONS1, true); /* <main>/Game/Server opt initial */
    menus_set_sensitive(game_menu,MENU_GAME_SERVER_OPTIONS2, true); /* <main>/Game/Server opt ongoing */
    menus_set_sensitive(game_menu,MENU_GAME_OUTPUT_LOG, true); /* <main>/Game/Export Log */
    menus_set_sensitive(game_menu,MENU_GAME_CLEAR_OUTPUT, true); /* <main>/Game/Clear Log */
    menus_set_sensitive(game_menu,MENU_GAME_DISCONNECT, true); /* <main>/Game/Disconnect */

  }
  else {
    struct unit *punit;
    menus_set_sensitive(report_menu,MENU_REPORT, true); /* <main>/Reports */
    menus_set_sensitive(kingdom_menu,MENU_KINGDOM, true); /* <main>/Kingdom */
    menus_set_sensitive(view_menu,MENU_VIEW, true); /* <main>/View */
    menus_set_sensitive(order_menu,MENU_ORDER, true); /* <main>/Orders */

    menus_set_sensitive(game_menu,MENU_GAME_OPTIONS, true); /* <main>/Game/Options */
    menus_set_sensitive(game_menu,MENU_GAME_MSG_OPTIONS, true); /* <main>/Game/Message Options */
    menus_set_sensitive(game_menu,MENU_GAME_SAVE_SETTINGS, true); /* <main>/Game/Save Settings */
    menus_set_sensitive(game_menu,MENU_GAME_PLAYERS, true); /* <main>/Game/Players */
    menus_set_sensitive(game_menu,MENU_GAME_MESSAGES, true); /* <main>/Game/Messages */
    menus_set_sensitive(game_menu,MENU_GAME_SERVER_OPTIONS1, true); /* <main>/Game/Server opt initial */
    menus_set_sensitive(game_menu,MENU_GAME_SERVER_OPTIONS2, true); /* <main>/Game/Server opt ongoing */
    menus_set_sensitive(game_menu,MENU_GAME_OUTPUT_LOG, true); /* <main>/Game/Export Log */
    menus_set_sensitive(game_menu,MENU_GAME_CLEAR_OUTPUT, true); /* <main>/Game/Clear Log */
    menus_set_sensitive(game_menu,MENU_GAME_DISCONNECT, true); /* <main>/Game/Disconnect */

    menus_set_sensitive(report_menu,MENU_REPORT_SPACESHIP,
			(game.player_ptr->spaceship.state!=SSHIP_NONE)); /* <main>/Reports/Spaceship */

    if((punit=get_unit_in_focus())) {
      char *irrtext  ="Build Irrigation",
	   *mintext  ="Build Mine",
	   *roadtext ="Build Road",
	   *transtext="Transform Terrain";
	  
      menus_set_sensitive(order_menu,MENU_ORDER_AUTO_SETTLER,
			  (can_unit_do_auto(punit)
			   && unit_flag(punit->type, F_SETTLERS))); /* <main>/Orders/Auto Settler */

      menus_set_sensitive(order_menu,MENU_ORDER_AUTO_ATTACK,
			  (can_unit_do_auto(punit)
			   && !unit_flag(punit->type, F_SETTLERS))); /* <main>/Orders/Auto-attack */

      menus_set_sensitive(order_menu,MENU_ORDER_CITY,
			   (can_unit_build_city(punit) ||
			    can_unit_add_to_city(punit))); /* <main>/Orders/Build City */
      menus_set_sensitive(order_menu,MENU_ORDER_FORTRESS,
			   can_unit_do_activity(punit, ACTIVITY_FORTRESS)); /* <main>/Orders/Build Fortress */
      menus_set_sensitive(order_menu,MENU_ORDER_ROAD,
			   can_unit_do_activity(punit, ACTIVITY_ROAD) ||
			   can_unit_do_activity(punit, ACTIVITY_RAILROAD)); /* <main>/Orders/Build Road */
      menus_set_sensitive(order_menu,MENU_ORDER_POLLUTION,
			   can_unit_do_activity(punit, ACTIVITY_POLLUTION)); /* <main>/Orders/Clean Pollution */
      menus_set_sensitive(order_menu,MENU_ORDER_FORTIFY,
			   can_unit_do_activity(punit, ACTIVITY_FORTIFY)); /* <main>/Orders/Fortify */
      menus_set_sensitive(order_menu,MENU_ORDER_SENTRY,
			   can_unit_do_activity(punit, ACTIVITY_SENTRY)); /* <main>/Orders/Sentry */
      menus_set_sensitive(order_menu,MENU_ORDER_PILLAGE,
			   can_unit_do_activity(punit, ACTIVITY_PILLAGE)); /* <main>/Orders/Pillage */
      menus_set_sensitive(order_menu,MENU_ORDER_EXPLORE,
			   can_unit_do_activity(punit, ACTIVITY_EXPLORE)); /* <main>/Orders/Auto-explore */
      menus_set_sensitive(order_menu,MENU_ORDER_MINE,
			   can_unit_do_activity(punit, ACTIVITY_MINE)); /* <main>/Orders/Build Mine */
      menus_set_sensitive(order_menu,MENU_ORDER_IRRIGATE,
			   can_unit_do_activity(punit, ACTIVITY_IRRIGATE)); /* <main>/Orders/Build Irrigation */
      menus_set_sensitive(order_menu,MENU_ORDER_TRANSFORM,
			   can_unit_do_activity(punit, ACTIVITY_TRANSFORM)); /* <main>/Orders/Transform Terrain */
      menus_set_sensitive(order_menu,MENU_ORDER_HOMECITY,
			   can_unit_change_homecity(punit)); /* <main>/Orders/Make Homecity */
      menus_set_sensitive(order_menu,MENU_ORDER_NUKE,
                           unit_flag(punit->type, F_NUCLEAR) &&
                           has_capability("nuke", aconnection.capability)); /* <main>/Orders/Explode Nuclear */
      menus_set_sensitive(order_menu,MENU_ORDER_UNLOAD,
			   get_transporter_capacity(punit)>0); /* <main>/Orders/Unload */
      menus_set_sensitive(order_menu,MENU_ORDER_WAKEUP, 
			   is_unit_activity_on_tile(ACTIVITY_SENTRY,
				punit->x, punit->y)); /* <main>/Orders/Wake up others */
      menus_set_sensitive(order_menu,MENU_ORDER_BUILD_WONDER,
			   unit_can_help_build_wonder_here(punit)); /* <main>/Orders/Help Build Wonder */
      menus_set_sensitive(order_menu,MENU_ORDER_TRADE_ROUTE,
			   unit_can_est_traderoute_here(punit)); /* <main>/Orders/Make Trade Route */
      if (unit_flag(punit->type, F_SETTLERS)
	 && map_get_city(punit->x, punit->y)) {
       menus_rename(order_menu, MENU_ORDER_CITY, (unsigned char *)"Add to City"); /* <main>/Orders/Build City */
      } else {
       menus_rename(order_menu, MENU_ORDER_CITY, (unsigned char *)"Build City"); /* <main>/Orders/Build City */
      }

      switch(map_get_tile(punit->x, punit->y)->terrain) {
      case T_ARCTIC:
	transtext="Change to Tundra";
	break;
      case T_DESERT:
	transtext="Change to Plains";
	break;
      case T_FOREST:
	irrtext  ="Change to Plains";
	transtext="Change to Grassland";
	break;
     case T_GRASSLAND:
	mintext  ="Change to Forest";
	transtext="Change to Hills";
	break;
      case T_HILLS:
	transtext="Change to Plains";
	break;
     case T_JUNGLE:
	irrtext  ="Change to Grassland";
	mintext  ="Change to Forest";
	transtext="Change to Plains";
       break;
      case T_MOUNTAINS:
	transtext="Change to Hills";
	break;
     case T_PLAINS:
	mintext  ="Change to Forest";
	transtext="Change to Grassland";
	break;
     case T_SWAMP:
	irrtext  ="Change to Grassland";
	mintext  ="Change to Forest";
	transtext="Change to Plains";
	break;
      case T_TUNDRA:
	transtext="Change to Desert";
	break;
      default:
	break;
      };

      menus_rename(order_menu, MENU_ORDER_IRRIGATE, (unsigned char *) irrtext); /* <main>/Orders/Build Irrigation */
      menus_rename(order_menu, MENU_ORDER_MINE, (unsigned char *) mintext); /* <main>/Orders/Build Mine */
      menus_rename(order_menu, MENU_ORDER_TRANSFORM, (unsigned char *) transtext); /* <main>/Orders/Transform Terrain */
    
      if (map_get_tile(punit->x,punit->y)->special&S_ROAD) {
	roadtext="Build Railroad";
	road_activity=ACTIVITY_RAILROAD;  
      } else {
	roadtext="Build Road";
	road_activity=ACTIVITY_ROAD;  
      }
      menus_rename(order_menu, MENU_ORDER_ROAD, (unsigned char *) roadtext); /* <main>/Orders/Build Road */
      menus_set_sensitive(order_menu, MENU_ORDER, TRUE);
    }
    else
      menus_set_sensitive(order_menu, MENU_ORDER, FALSE);
  }
}
